#include "pluginhandler.h"
#include "coffeeworkflowcontroller.h"
#include "pluginscontainer.h"
#include <QQmlContext>

#include <coffeeplugin.h>

PluginHandler::PluginHandler(PluginsContainer *pluginsContainer,
                             QQmlContext *context, QObject *parent)
    : QObject{parent}, m_pluginsContainer(pluginsContainer),
      m_qmlContext(context), m_currentCoffeeWorkflowController(nullptr) {}

QUrl PluginHandler::currentScreen() const { return m_currentScreen; }

void PluginHandler::setCurrentScreen(const QUrl &newCurrentScreen) {
  if (m_currentScreen == newCurrentScreen)
    return;
  m_currentScreen = newCurrentScreen;
  emit currentScreenChanged();
}

void PluginHandler::handlePlugin(const QString &pluginName) {

  CoffeePlugin *coffeePlugin = m_pluginsContainer->coffeePlugin(pluginName);

  m_currentCoffeeWorkflowController =
      coffeePlugin->createCoffeeWorkFlowController();

  if (m_currentCoffeeWorkflowController) {

    m_qmlContext->setContextProperty(
        "controller", m_currentCoffeeWorkflowController->qmlPresenterData());

    m_currentCoffeeWorkflowController->setBeginUiWorkflowCallback([this]() {
      setCurrentScreen(m_currentCoffeeWorkflowController->mainPanelQmlUrl());
    });

    m_currentCoffeeWorkflowController->setEndUiWorkflowCallback([this]() {
      emit pluginDone();
      m_currentCoffeeWorkflowController->destroy();
      m_currentCoffeeWorkflowController = nullptr;
    });

    m_currentCoffeeWorkflowController->startWork(QVariantMap());
  }
}

// CoffeePlugin *pluginCon
