#include "pluginscontainer.h"
#include "coffeeplugin.h"
#include <QDebug>
#include <QDir>
#include <QPluginLoader>

PluginsContainer::PluginsContainer(QObject *parent) : QObject{parent} {

  populateContainer();
}

void PluginsContainer::populateContainer() {

  QDir workspace(COFFEE_PLUGIN_PATH);
  QStringList contents =
      workspace.entryList(QDir::Dirs | QDir::NoDotDot | QDir::NoDot);

  for (auto pluginDir : contents) {
    QDir currentPluginDir = workspace;
    currentPluginDir.cd(pluginDir);
    const QStringList entries =
        currentPluginDir.entryList({QStringLiteral("*.so")}, QDir::Files);

    if (entries.count() == 1) {
      QPluginLoader *pluginLoader = new QPluginLoader(
          currentPluginDir.absoluteFilePath(entries.at(0)), this);

      QObject *pluginElement = pluginLoader->instance();
      if (pluginElement) {

        CoffeePlugin *coffeePlugin =
            qobject_cast<CoffeePlugin *>(pluginElement);

        if (coffeePlugin) {
          m_container.insert(pluginDir, coffeePlugin);
        }
      }
    }
  }
}

int PluginsContainer::pluginsCount() const { return m_container.count(); }

QString PluginsContainer::pluginName(int index) const {

  QString result;

  if (index < m_container.count()) {
    result = m_container.keys().at(index);
  }

  return result;
}

QUrl PluginsContainer::pluginIcon(const QString &pluginName) const {

  CoffeePlugin *plugin = coffeePlugin(pluginName);

  if (plugin) {
    return plugin->getCoffeeIcon();
  } else
    return QUrl();
}

CoffeePlugin *PluginsContainer::coffeePlugin(const QString &pluginName) const {

  return m_container.value(pluginName, nullptr);
}
