#pragma once

#include <QObject>
#include <QUrl>

#include <memory>

class CoffeeWorkflowController {
public:
  CoffeeWorkflowController() {}

  virtual void startWork(const QVariantMap &) = 0;

  virtual QUrl mainPanelQmlUrl() const = 0;
  virtual QObject *qmlPresenterData() const = 0;

  void beginUiWorkflow();
  void endUiWorkFlow();

  virtual void destroy() = 0;

  void
  setBeginUiWorkflowCallback(std::function<void()> beginUiWorkflowCallback);
  void setEndUiWorkflowCallback(std::function<void()> doneCallback);

protected:
  std::function<void(void)> m_beginUiWorkflowCallback;
  std::function<void(void)> m_endUiworkflowCallback;
};

inline void CoffeeWorkflowController::setBeginUiWorkflowCallback(
    std::function<void()> beginUiWorkflowCallback) {
  m_beginUiWorkflowCallback = beginUiWorkflowCallback;
}

inline void CoffeeWorkflowController::setEndUiWorkflowCallback(
    std::function<void()> doneCallback) {
  m_endUiworkflowCallback = doneCallback;
}

inline void CoffeeWorkflowController::beginUiWorkflow() {
  if (m_beginUiWorkflowCallback) {
    m_beginUiWorkflowCallback();
  }
}

inline void CoffeeWorkflowController::endUiWorkFlow() {
  if (m_endUiworkflowCallback) {
    m_endUiworkflowCallback();
  }
}
